@@ -35,4 +35,8 @@ HOME_REGION = os.environ.get('AWS_REGION', os.environ.get('AWS_DEFAULT_REGION',
 EVENTBRIDGE_MAX_EVENTS_PER_CALL = 10
 
 SPOKE_VERSION_SSM_PARAM_NAME = 'service-catalog-puppet-spoke-version'
-PUPPET_VERSION_SSM_PARAM_NAME = 'service-catalog-puppet-version'
\ No newline at end of file
+PUPPET_VERSION_SSM_PARAM_NAME = 'service-catalog-puppet-version'
+
+
+SPOKE_LOCAL_PORTFOLIO_STATUS_SHARED = "shared"
+SPOKE_LOCAL_PORTFOLIO_STATUS_TERMINATED = "terminated"
\ No newline at end of file
